"""
SAX driver for rxpop C module.
"""

from xml.sax.xmlreader import XMLReader, Locator
from xml.sax import SAXReaderNotAvailable

import urllib

try:
    from xml.parsers import rxpop
except ImportError:
    raise SAXReaderNotAvailable("rxpop not supported", None)


class SAXRxpopParser(XMLReader, Locator):
    """
    SAX driver for rxpop C module.
    """
    def __init__(self):
        XMLReader.__init__(self)
        self.parser = rxpop.ParserCreate()
        self.contentHandler = None
        self.documentLocator = None

    def get_parser_name(self):
        return 'rxpop'

    def get_parser_version(self):
        return '1.0a'

    def get_driver_version(self):
        return rxpop.GetDriverVersion()

    def parse(self, sysID):
        # If the contentHandler has a documentLocator method,
        #   create a DocumentLocator and pass it.
        if self.contentHandler and \
            hasattr(self.contentHandler, 'setDocumentLocator') and \
            callable(self.contentHandler.setDocumentLocator):
            self.documentLocator = DocumentLocator(self.parser)
            self.contentHandler.setDocumentLocator(self.documentLocator)
        self.parser.parse(sysID)
 
    def parseFile(self, fileobj):
        self.parser.parseFile()

    def setContentHandler(self, handler):
        XMLReader.setContentHandler(self, handler)
        self.parser.setContentHandler(handler)

    def setDTDHandler(self, handler):
        XMLReader.setDTDHandler(self, handler)
        self.parser.setDTDHandler(handler)

    def setErrorHandler(self, handler):
        XMLReader.setErrorHandler(self, handler)
        self.parser.setErrorHandler(handler)

    def activateEventHandler(self, handlerName, flag):
        self.parser.activate(handlerName, flag)

    def setParserFlag(self, flagName, flag):
        return self.parser.setParserFlag(flagName, flag)

    def getParserFlag(self, flagName):
        return self.parser.getParserFlag(flagName)

    def getLocation(self):
        return self.parser.getLocation()

    def getFeature(self, featurename):
        if name == saxlib.feature_validation:
            flag = self.parser.getParserFlag('Validate')
            return flag
        elif name == saxlib.feature_namespaces:
            flag = self.parser.getParserFlag('XMLNamespaces')
            return flag
        else:
            raise saxlib.SAXNotRecognizedException(
                "Feature '%s' not recognized" % featurename)

    def setFeature(self, featurename, value):
        if name == saxlib.feature_validation:
            if value:
                self.parser.setParserFlag('Validate', 1)
            else:
                self.parser.setParserFlag('Validate', 0)
        elif name == saxlib.feature_namespaces:
            if value:
                self.parser.setParserFlag('XMLNamespaces', 1)
            else:
                self.parser.setParserFlag('XMLNamespaces', 0)
        else:
            raise saxlib.SAXNotRecognizedException(
                "Feature '%s' not recognized" % featurename)


    def getProperty(self, propertyname):
        pass

    def setProperty(self, propertyname, value):
        pass


class DocumentLocator:
    def __init__(self, parser):
        self.parser = parser
    def getLocation(self):
        "Return a tuple: (lineNum, columnNum)."
        return self.parser.getLocation()


def create_parser():
    return SAXRxpopParser()

